function [U_full,Lambda,output_k,output_res]=primal_dual_method(Z_0,lambda)

global m n f_K blur_p blur_para std_dev             % degraded image information
global r mu gamma bc                                    % primal-dual model
global local_var_est om zeta kmax                % update lambda
global ls_parameter res_tol imax i_p cg_imax pk_p pk_para % Newton steps


%% ---------- Initialize and setting storage --------------------------------
Z=Z_0;
s=1/(1-1/r);   

res_ol=zeros(kmax,1);

i=1;
k=1;

Lambda=lambda*ones(m,n);
Lambda_t=Lambda;

%vp=0;  visual_pic; % visualize initial pictures

if local_var_est==1
    lve_b=lve_bound(m*n,om^2);  
else
    kmax=2;
end


%% Primal dual method 
while k<kmax
    
    % initialization
    %U=smooth_filter(Z,[11,1],'g');
    U=zeros(m,n);
    P=zeros(m,n,2); 
    [P,num_infeasible_P]=correction_P(P,s);
        
    res=zeros(imax,1);
    fvalue=zeros(imax,1);
    fvalue(1)=function_value(U,Z,Lambda,gamma,mu,r,f_K,blur_para,0,bc);
    uv=UV(U,P,gamma,r,bc); %useful variables
    res(1)=res_func(U,P,Z,uv,Lambda,r,f_K,blur_para,mu,bc);
    inner_tol=1e-4;
    
    
    while i<imax     
        
        f=rs(U,Z,uv,Lambda,mu,r,f_K,blur_para,bc);
    
        % Newton solver
        DU=comp_du(uv,f,Lambda,f_K,blur_para,inner_tol,m,n,i_p,pk_p,blur_p, cg_imax,pk_para);
          
        
        % Line Search 
        if ls_parameter==1
            
            ls_a=0.8;
            ls_itermax=500;
            
            DP=comp_dp(DU,P,uv,r,bc);
          
            ls_ai=line_search(U,DU,Z,Lambda,gamma,mu,r,f_K,blur_para,ls_a,ls_itermax,bc);
           
            DU=ls_ai*DU;
            DP=ls_ai*DP;
        else
            DP=comp_dp(DU,P,uv,r,bc);
        end
        
        
        % Update u and p
        P=P+DP;
        U=U+DU;
        
           
        % Visualize 
        % vp=1; visual_pic;
        
        i=i+1;
        
        fvalue(i)=function_value(U,Z,Lambda,gamma,mu,r,f_K,blur_para,0,bc);
                                
        [P,num_infeasible_P(i)]=correction_P(P,s);
        
        uv=UV(U,P,gamma,r,bc); %useful variables
        
        res(i)=res_func(U,P,Z,uv,Lambda,r,f_K,blur_para,mu,bc);

        inner_tol=0.1*min((res(i)/res(1))^(3/2),res(i)/res(1));
            
        % Stopping Rule 
        if res(i)/res(1)<res_tol || norm(DU,'fro')<1e-8, break; end
                
        
    end
    

    % The residual
    
    res_image=(feval(f_K,U,blur_para,m,n)-Z).^2;
    
    
    % Multiscale Decomposition 
    if k==1;
        U_full=U;
    else
        U_full=U_full+U;
    end
    
    if local_var_est==1    
        
        KU_full=feval(f_K,U_full,blur_para,m,n);
        Z=Z_0-KU_full;
     
        % Stopping Rule 
        iend=0;
        i=1;

        res_ol(k)=norm(Z_0-KU_full,'fro')^2/(n*m);

        if res_ol(k)<std_dev^2,  break;  end


        % Local Variance estimator (Lambda) 
        LVE=smooth_filter(res_image,om,'m');    % mean filter
        B_lve=lve_b*std_dev^2;

        roh=norm(Lambda_t(:),'inf')/std_dev;

        I_LVE=LVE>=B_lve;

        LVEtilde=I_LVE.*LVE+(1-I_LVE).*std_dev^2;

        Lambda_t=zeta*(Lambda_t+roh*(LVEtilde-std_dev^2));

        Lambda=smooth_filter(Lambda_t,om,'m');    


        % Visualization 
        % vp=2; visual_pic;
    end
    
    
    k=k+1;
end

if local_var_est==1
    output_k=k-1;
    output_res=res_ol(1:k-1);
else
    output_k=i-1;
    output_res=res(1:i-1);
end


